package com.qmth.qrzk.repository.user.model;

import java.util.Date;
import java.util.List;

import com.qmth.qrzk.repository.common.enumeration.QuestionStatusEnum;
import com.qmth.qrzk.repository.task.PaperTask;
import com.qmth.qrzk.repository.utils.Consts;
import com.qmth.qrzk.repository.utils.DateHelper;
import com.qmth.qrzk.repository.utils.General;

/**
 * 试题实体类
 * 
 * @author 钟辉
 * 
 */
public class Question {

    private Integer id; // 编号

    private String lutiType; // 录题类型，tika为题卡录题；mingti为命题录题，用于与客户端交互

    private String questionNo; // 题号

    private Integer courseId; // 课程id

    private String courseName; // 课程名称
    
    private String courseNo; // 课程编号

    private Integer typeId; // 题型id

    private String typeName; // 题型名称

    private Integer bookId; // 教材id

    private String bookName; // 将bookId转换成对应的bookName

    private Integer batchId; // 批次id

    private String estimateDifficultyCode; // 预计难度

    private String estimateDifficultyText; // 预计难度显示值

    private String measuredDifficultyCode; // 实测难度

    private String measuredDifficultyText; // 实测难度显示值

    private String gradeCode; // 层次

    private String gradeText; // 层次显示值

    private String abilityCode; // 认知层次

    private String abilityText; // 认知层次显示值

    private String chapter; // 章

    private String section; // 节

    private String item; // 目

    private String page; // 页

    private Double score; // 满分值

    private byte[] bodyWord; // 题干word;

    private String body; // 题干

    private String bodySummary; // 题干摘要

    private byte[] answerWord; // 答案word

    private String answer; // 答案

    private String answerSummary; // 答案摘要

    private String answerSheet; // 题卡

    private String statusCode; // 状态

    private Date enableTime; // 启用日期

    private String enableTimeStr; // 启用日期的字符串，用户客户端接口接收数据
    
    private String openTimesStr; // 历史启用日期拼接的字符串

    private Date setTime; // 命题时间

    private Integer setBy; // 命题人

    private String setTeacherName; // 命题人名称

    private String remark; // 备注

    private Date createdTime; // 创建日期

    private Integer createdBy; // 创建人

    private User createdByBean;

    private Date updatedTime; // 更新日期

    private Integer updatedBy; // 更新人

    private User updatedByBean;

    private Integer version; // 版本

    private Integer importId; // 导入版本id

    private String keyword; // 查询关键字

    private int isCheck; // 是否选中 0表示未选中 ,1表示选中

    private int count; // 题量的count

    private String assessmentDemand; // 考核要求

    private String assessmentDemandText; // 考核要求显示值

    private List<QuestionOption> options;

    private List<OperLog> auditLogs; // 试题的审核日志

    private List<PaperTask> paperTasks; // 关联的试卷列表

    private Date enableDate; // 试题被启用的日期

    private String guid;
    
    private byte[] editWord; //题干Word数据、选项word、答案Word数据
    
    private String questionName;//试题名称：选择题1，计算试卷相似度时需要用到

    public String getGuid() {
        return guid;
    }

    public void setGuid(String guid) {
        this.guid = guid;
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    public String getAnswerSheet() {
        return answerSheet;
    }

    public void setAnswerSheet(String answerSheet) {
        this.answerSheet = answerSheet;
    }

    public String getOpenTimesStr() {
		return openTimesStr;
	}

	public void setOpenTimesStr(String openTimesStr) {
		this.openTimesStr = openTimesStr;
	}

	public byte[] getEditWord() {
		return editWord;
	}

	public void setEditWord(byte[] editWord) {
		this.editWord = editWord;
	}

	public String getBookName() {
        return bookName;
    }

    public void setBookName(String bookName) {
        this.bookName = bookName;
    }

    public String getKeyword() {
        return keyword;
    }

    public void setKeyword(String keyword) {
        this.keyword = keyword;
    }

    public String getAssessmentDemandText() {
        return General.null2String(Consts.CodeTextMap.get("ASSESSMENT_DEMAND-" + assessmentDemand));
    }

    public void setAssessmentDemandText(String assessmentDemandText) {
        this.assessmentDemandText = assessmentDemandText;
    }

    public String getAssessmentDemand() {
        return assessmentDemand;
    }

    public int getIsCheck() {
        return isCheck;
    }

    public void setIsCheck(int isCheck) {
        this.isCheck = isCheck;
    }

    public void setAssessmentDemand(String assessmentDemand) {
        this.assessmentDemand = assessmentDemand;
    }

    public Question(Integer id, String questionNo, Integer courseId, Integer typeId, Integer bookId, Integer batchId,
            String estimateDifficultyCode, String measuredDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, double score, byte[] bodyWord, String body,
            String bodySummary, byte[] answerWord, String answer, String answerSummary, String answerSheet,
            String statusCode, Date enableTime, Date setTime, Integer setBy, String setTeacherName, String remark,
            Integer createdBy, Integer updatedBy) {
        super();
        this.id = id;
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.answerSheet = answerSheet;
        this.statusCode = statusCode;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdBy = createdBy;
        this.updatedBy = updatedBy;
    }

    public Question(Integer id, String questionNo, Integer courseId, Integer typeId, Integer bookId, Integer batchId,
            String estimateDifficultyCode, String measuredDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, double score, byte[] bodyWord, String body,
            String bodySummary, byte[] answerWord, String answer, String answerSummary, String statusCode,
            Date enableTime, Date setTime, Integer setBy, String setTeacherName, String remark, Date createdTime,
            Integer createdBy, Date updatedTime, Integer updatedBy, Integer version, Integer importId) {
        super();
        this.id = id;
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.statusCode = statusCode;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdTime = createdTime;
        this.createdBy = createdBy;
        this.updatedTime = updatedTime;
        this.updatedBy = updatedBy;
        this.version = version;
        this.importId = importId;
    }

    public Question(String questionNo, Integer courseId, Integer typeId, Integer bookId, Integer batchId,
            String estimateDifficultyCode, String measuredDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, double score, byte[] bodyWord, String body,
            String bodySummary, byte[] answerWord, String answer, String answerSummary, String answersheet,
            String statusCode, Date enableTime, Date setTime, Integer setBy, String setTeacherName, String remark,
            Date createdTime, Integer createdBy, Date updatedTime, Integer updatedBy, Integer version, Integer importId) {
        super();
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.answerSheet = answersheet;
        this.statusCode = statusCode;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdTime = createdTime;
        this.createdBy = createdBy;
        this.updatedTime = updatedTime;
        this.updatedBy = updatedBy;
        this.version = version;
        this.importId = importId;
    }

    public Integer getImportId() {
        return importId;
    }

    public void setImportId(Integer importId) {
        this.importId = importId;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public String getQuestionNo() {
        return questionNo;
    }

    public String getTypeName() {
        return typeName;
    }

    public void setTypeName(String typeName) {
        this.typeName = typeName;
    }

    public void setQuestionNo(String questionNo) {
        this.questionNo = questionNo;
    }

    public Integer getCourseId() {
        return courseId;
    }

    public void setCourseId(Integer courseId) {
        this.courseId = courseId;
    }

    public Integer getTypeId() {
        return typeId;
    }

    public void setTypeId(Integer typeId) {
        this.typeId = typeId;
    }

    public Integer getBookId() {
        return bookId;
    }

    public void setBookId(Integer bookId) {
        this.bookId = bookId;
    }

    public Integer getBatchId() {
        return batchId;
    }

    public void setBatchId(Integer batchId) {
        this.batchId = batchId;
    }

    public String getEstimateDifficultyCode() {
        return estimateDifficultyCode;
    }

    public void setEstimateDifficultyCode(String estimateDifficultyCode) {
        this.estimateDifficultyCode = estimateDifficultyCode;
    }

    public String getMeasuredDifficultyCode() {
        return measuredDifficultyCode;
    }

    public void setMeasuredDifficultyCode(String measuredDifficultyCode) {
        this.measuredDifficultyCode = measuredDifficultyCode;
    }

    public String getGradeCode() {
        return gradeCode;
    }

    public void setGradeCode(String gradeCode) {
        this.gradeCode = gradeCode;
    }

    public String getAbilityCode() {
        return abilityCode;
    }

    public void setAbilityCode(String abilityCode) {
        this.abilityCode = abilityCode;
    }

    public String getChapter() {
        return chapter;
    }

    public void setChapter(String chapter) {
        this.chapter = chapter;
    }

    public String getSection() {
        return section;
    }

    public void setSection(String section) {
        this.section = section;
    }

    public String getItem() {
        return item;
    }

    public void setItem(String item) {
        this.item = item;
    }

    public String getPage() {
        return page;
    }

    public void setPage(String page) {
        this.page = page;
    }

    public Double getScore() {
        return score;
    }

    public void setScore(Double score) {
        this.score = score;
    }

    public byte[] getBodyWord() {
        return bodyWord;
    }

    public void setBodyWord(byte[] bodyWord) {
        this.bodyWord = bodyWord;
    }

    public String getBody() {
        return body;
    }

    public void setBodyHtml(String str) {
        // do nothing
    }

    public String getBodyHtml() {
        return (body != null && !body.equals("")) ? body : bodySummary;
    }

    public void setBody(String body) {
        this.body = body;
    }

    public String getBodySummary() {
        return bodySummary;
    }

    public void setBodySummary(String bodySummary) {
        this.bodySummary = bodySummary;
    }

    public byte[] getAnswerWord() {
        return answerWord;
    }

    public void setAnswerWord(byte[] answerWord) {
        this.answerWord = answerWord;
    }

    public String getAnswer() {
        return answer;
    }

    public void setAnswerHtml(String str) {
        // do nothing
    }

    public String getAnswerHtml() {
        return (answer != null && !answer.equals("")) ? answer : answerSummary;
    }

    public void setAnswer(String answer) {
        this.answer = answer;
    }

    public String getAnswerSummary() {
        return answerSummary;
    }

    public void setAnswerSummary(String answerSummary) {
        this.answerSummary = answerSummary;
    }

    public String getStatusCode() {
        return statusCode;
    }

    public void setStatusCode(String statusCode) {
        this.statusCode = statusCode;
    }

    public Date getEnableTime() {
        return enableTime;
    }

    public void setEnableTimeStr(String enableTimeStr) {
        this.enableTimeStr = enableTimeStr;
    }

    public String getEnableTimeStr() {
        if (enableTimeStr != null && enableTimeStr.length() > 0) {
            return enableTimeStr;
        }
        return enableTime == null ? null : DateHelper.toShortString(enableTime);
    }

    public void setEnableTime(Date enableTime) {
        this.enableTime = enableTime;
    }

    public Date getSetTime() {
        return setTime;
    }

    public void setSetTime(Date setTime) {
        this.setTime = setTime;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public Date getUpdatedTime() {
        return updatedTime;
    }

    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    public Integer getSetBy() {
        return setBy;
    }

    public void setSetBy(Integer setBy) {
        this.setBy = setBy;
    }

    public String getSetTeacherName() {
        return setTeacherName;
    }

    public void setSetTeacherName(String setTeacherName) {
        this.setTeacherName = setTeacherName;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public Integer getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(Integer createdBy) {
        this.createdBy = createdBy;
    }

    public Integer getUpdatedBy() {
        return updatedBy;
    }

    public void setUpdatedBy(Integer updatedBy) {
        this.updatedBy = updatedBy;
    }

    public Integer getId() {
        return id;
    }

    public Question(String questionNo, Integer courseId, Integer typeId, Integer bookId, Integer batchId,
            String estimateDifficultyCode, String measuredDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, double score, byte[] bodyWord, String body,
            String bodySummary, byte[] answerWord, String answer, String answerSummary, String answerSheet,
            String statusCode, Date enableTime, Date setTime, Integer setBy, String setTeacherName, String remark,
            Date createdTime, Integer createdBy, Date updatedTime, Integer updatedBy) {
        super();
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.answerSheet = answerSheet;
        this.statusCode = statusCode;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdTime = createdTime;
        this.createdBy = createdBy;
        this.updatedTime = updatedTime;
        this.updatedBy = updatedBy;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Question(Integer bookId, String estimateDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, String statusCode) {
        super();
        this.bookId = bookId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.statusCode = statusCode;
    }

    public Question() {
        super();
        // TODO Auto-generated constructor stub
    }

    public Question(Integer courseId, Integer typeId, Integer bookId, String estimateDifficultyCode,
            String assessmentDemand, String abilityCode, String chapter, String section, String item, String page,
            double score, String body, String bodySummary, String answer, String answerSummary, Date enableTime,
            Date setTime, Integer setBy, String setTeacherName, String remark, Date createdTime, Integer createdBy,
            Date updatedTime, Integer updatedBy) {
        super();
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdTime = createdTime;
        this.createdBy = createdBy;
        this.updatedTime = updatedTime;
        this.updatedBy = updatedBy;
    }

    public List<QuestionOption> getOptions() {
        return options;
    }

    public void setOptions(List<QuestionOption> options) {
        this.options = options;
    }

    public Question(String questionNo, Integer courseId, Integer typeId, Integer bookId, String estimateDifficultyCode,
            String measuredDifficultyCode, String assessmentDemand, String abilityCode, String chapter, String section,
            String item, String page, double score, byte[] bodyWord, String body, String bodySummary,
            byte[] answerWord, String answer, String answerSummary, String answerSheet, String statusCode,
            Date enableTime, Date setTime, Integer setBy, String setTeacherName, String remark, Date createdTime,
            Integer createdBy, Date updatedTime, Integer updatedBy) {
        super();
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.answerSheet = answerSheet;
        this.statusCode = statusCode;
        this.enableTime = enableTime;
        this.setTime = setTime;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdTime = createdTime;
        this.createdBy = createdBy;
        this.updatedTime = updatedTime;
        this.updatedBy = updatedBy;
    }

    public Question(Integer id, Integer courseId, Integer typeId, Integer bookId, String estimateDifficultyCode,
            String assessmentDemand, String abilityCode, String chapter, String section, String item, String page,
            String bodySummary, String answerSummary, String statusCode, Integer setBy, Integer createdBy) {
        super();
        this.id = id;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.bodySummary = bodySummary;
        this.answerSummary = answerSummary;
        this.statusCode = statusCode;
        this.setBy = setBy;
        this.createdBy = createdBy;
    }

    public Question(Integer id, String questionNo, Integer courseId, Integer typeId, Integer bookId, Integer batchId,
            String estimateDifficultyCode, String measuredDifficultyCode, String assessmentDemand, String abilityCode,
            String chapter, String section, String item, String page, double score, byte[] bodyWord, String body,
            String bodySummary, byte[] answerWord, String answer, String answerSummary, String statusCode,
            Integer setBy, String setTeacherName, String remark, Integer createdBy, Integer updatedBy) {
        super();
        this.id = id;
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.assessmentDemand = assessmentDemand;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.statusCode = statusCode;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdBy = createdBy;
        this.updatedBy = updatedBy;
    }

    public String getEstimateDifficultyText() {
        return General.null2String(Consts.CodeTextMap.get("QUESTION_DIFFICULTY-" + estimateDifficultyCode));
    }

    public void setEstimateDifficultyText(String estimateDifficultyText) {
        this.estimateDifficultyText = estimateDifficultyText;
    }

    public String getMeasuredDifficultyText() {
        return General.null2String(Consts.CodeTextMap.get("QUESTION_DIFFICULTY-" + measuredDifficultyCode));
    }

    public void setMeasuredDifficultyText(String measuredDifficultyText) {
        this.measuredDifficultyText = measuredDifficultyText;
    }

    public String getGradeText() {
        return General.null2String(Consts.CodeTextMap.get("GRADE-" + gradeCode));
    }

    public void setGradeText(String gradeText) {
        this.gradeText = gradeText;
    }

    public String getAbilityText() {
        return General.null2String(Consts.CodeTextMap.get("ABILITY-" + abilityCode));
    }

    public void setAbilityText(String abilityText) {
        this.abilityText = abilityText;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public void setStatusName(String str) {
        // do nothing
    }

    public String getStatusName() {
        if (statusCode == null) {
            return "未知状态";
        }
        QuestionStatusEnum sEnum = QuestionStatusEnum.valueOf(statusCode);
        return sEnum != null ? sEnum.getValue() : "未知状态";
    }

    public List<OperLog> getAuditLogs() {
        return auditLogs;
    }

    public void setAuditLogs(List<OperLog> auditLogs) {
        this.auditLogs = auditLogs;
    }

    public Question(Integer id, String questionNo, Integer courseId, String courseName, Integer typeId,
            String typeName, Integer bookId, String bookName, Integer batchId, String estimateDifficultyCode,
            String measuredDifficultyCode, String assessmentDemand, String abilityCode, String chapter, String section,
            String item, String page, double score, byte[] bodyWord, String body, String bodySummary,
            byte[] answerWord, String answer, String answerSummary, String statusCode, Integer setBy,
            String setTeacherName, String remark, Integer createdBy, Integer updatedBy) {
        super();
        this.id = id;
        this.questionNo = questionNo;
        this.courseId = courseId;
        this.courseName = courseName;
        this.typeId = typeId;
        this.typeName = typeName;
        this.bookId = bookId;
        this.bookName = bookName;
        this.batchId = batchId;
        this.estimateDifficultyCode = estimateDifficultyCode;
        this.measuredDifficultyCode = measuredDifficultyCode;
        this.abilityCode = abilityCode;
        this.chapter = chapter;
        this.section = section;
        this.item = item;
        this.page = page;
        this.score = score;
        this.bodyWord = bodyWord;
        this.body = body;
        this.bodySummary = bodySummary;
        this.answerWord = answerWord;
        this.answer = answer;
        this.answerSummary = answerSummary;
        this.statusCode = statusCode;
        this.setBy = setBy;
        this.setTeacherName = setTeacherName;
        this.remark = remark;
        this.createdBy = createdBy;
        this.updatedBy = updatedBy;
        this.assessmentDemand = assessmentDemand;
    }

    public Question(Integer courseId, Integer typeId, Integer bookId) {
        super();
        this.courseId = courseId;
        this.typeId = typeId;
        this.bookId = bookId;
    }

    public String getLutiType() {
        return lutiType;
    }

    public void setLutiType(String lutiType) {
        this.lutiType = lutiType;
    }

    public User getCreatedByBean() {
        return createdByBean;
    }

    public void setCreatedByBean(User createdByBean) {
        this.createdByBean = createdByBean;
    }

    public User getUpdatedByBean() {
        return updatedByBean;
    }

    public void setUpdatedByBean(User updatedByBean) {
        this.updatedByBean = updatedByBean;
    }

    public List<PaperTask> getPaperTasks() {
        return paperTasks;
    }

    public void setPaperTasks(List<PaperTask> paperTasks) {
        this.paperTasks = paperTasks;
    }

    public Date getEnableDate() {
        return enableDate;
    }

    public void setEnableDate(Date enableDate) {
        this.enableDate = enableDate;
    }

	public String getCourseNo() {
		return courseNo;
	}

	public void setCourseNo(String courseNo) {
		this.courseNo = courseNo;
	}

	public String getQuestionName() {
		return questionName;
	}

	public void setQuestionName(String questionName) {
		this.questionName = questionName;
	}
	
}
